"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _TransactionStack_stack, _TransactionStack_onPush, _TransactionStack_events, _TransactionStack_disposed;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionStack = exports.TransactionResult = void 0;
const client_utils_1 = require("@fluid-internal/client-utils");
const internal_1 = require("@fluidframework/telemetry-utils/internal");
/**
 * Describes the result of a transaction.
 * Transactions may either succeed and commit, or fail and abort.
 */
var TransactionResult;
(function (TransactionResult) {
    /**
     * Indicates the transaction failed.
     */
    TransactionResult[TransactionResult["Abort"] = 0] = "Abort";
    /**
     * Indicates the transaction succeeded.
     */
    TransactionResult[TransactionResult["Commit"] = 1] = "Commit";
})(TransactionResult || (exports.TransactionResult = TransactionResult = {}));
/**
 * An implementation of {@link Transactor} that uses a stack to manage transactions.
 * @remarks Using a stack allows transactions to nest - i.e. an inner transaction may be started while an outer transaction is already in progress.
 */
class TransactionStack {
    get events() {
        return __classPrivateFieldGet(this, _TransactionStack_events, "f");
    }
    get disposed() {
        return __classPrivateFieldGet(this, _TransactionStack_disposed, "f");
    }
    /**
     * Construct a new {@link TransactionStack}.
     * @param onPush - A {@link OnPush | function} that will be called when a transaction begins.
     */
    constructor(onPush) {
        _TransactionStack_stack.set(this, []);
        _TransactionStack_onPush.set(this, void 0);
        _TransactionStack_events.set(this, (0, client_utils_1.createEmitter)());
        _TransactionStack_disposed.set(this, false);
        __classPrivateFieldSet(this, _TransactionStack_onPush, onPush, "f");
    }
    isInProgress() {
        this.ensureNotDisposed();
        return __classPrivateFieldGet(this, _TransactionStack_stack, "f").length > 0;
    }
    start() {
        this.ensureNotDisposed();
        __classPrivateFieldGet(this, _TransactionStack_stack, "f").push(__classPrivateFieldGet(this, _TransactionStack_onPush, "f")?.call(this));
        __classPrivateFieldGet(this, _TransactionStack_events, "f").emit("started");
    }
    commit() {
        this.ensureNotDisposed();
        if (!this.isInProgress()) {
            throw new internal_1.UsageError("No transaction to commit");
        }
        __classPrivateFieldGet(this, _TransactionStack_events, "f").emit("committing");
        __classPrivateFieldGet(this, _TransactionStack_stack, "f").pop()?.(TransactionResult.Commit);
    }
    abort() {
        this.ensureNotDisposed();
        if (!this.isInProgress()) {
            throw new internal_1.UsageError("No transaction to abort");
        }
        __classPrivateFieldGet(this, _TransactionStack_events, "f").emit("aborting");
        __classPrivateFieldGet(this, _TransactionStack_stack, "f").pop()?.(TransactionResult.Abort);
    }
    dispose() {
        this.ensureNotDisposed();
        while (this.isInProgress()) {
            this.abort();
        }
        __classPrivateFieldSet(this, _TransactionStack_disposed, true, "f");
    }
    ensureNotDisposed() {
        if (this.disposed) {
            throw new internal_1.UsageError("Transactor is disposed");
        }
    }
}
exports.TransactionStack = TransactionStack;
_TransactionStack_stack = new WeakMap(), _TransactionStack_onPush = new WeakMap(), _TransactionStack_events = new WeakMap(), _TransactionStack_disposed = new WeakMap();
//# sourceMappingURL=transaction.js.map